@@ -26,6 +26,7 @@ from types import FunctionType, MethodType
 from typing import (Any, AsyncGenerator, Callable, Dict, Generator, Generic,
                     List, Optional, Sequence, Set, Tuple, Type, TypeVar, cast)
 from weakref import ref as weak_ref
+from weakref import ref as weakref
 
 from .dbus_common import (DbusMethodCommon, DbusSomethingAsync,
                           DbusSomethingSync, _method_name_converter,
@@ -34,6 +35,7 @@ from .dbus_exceptions import DbusFailedError
 from .sd_bus_internals import (DbusNoReplyFlag, SdBus, SdBusInterface,
                                SdBusMessage)
 
+
 T_input = TypeVar('T_input')
 T_result = TypeVar('T_result')
 T_obj = TypeVar('T_obj')
@@ -56,19 +58,25 @@ class DbusMethodAsyncBinded(DbusBindedAsync):
                  dbus_method: DbusMethodAsync,
                  interface: DbusInterfaceBaseAsync):
         self.dbus_method = dbus_method
-        self.interface = interface
+        self.interface_ref = weakref(interface)
+
+        self.__doc__ = dbus_method.__doc__
 
     async def _call_dbus_async(self, *args: Any) -> Any:
-        assert self.interface.attached_bus is not None
-        assert self.interface.remote_service_name is not None
-        assert self.interface.remote_object_path is not None
+        interface = self.interface_ref()
+        assert interface is not None
+
+        assert interface._attached_bus is not None
+        assert interface._remote_service_name is not None
+        assert interface._remote_object_path is not None
         assert self.dbus_method.interface_name is not None
-        new_call_message = self.interface.attached_bus.new_method_call_message(
-            self.interface.remote_service_name,
-            self.interface.remote_object_path,
-            self.dbus_method.interface_name,
-            self.dbus_method.method_name,
-        )
+        new_call_message = interface._attached_bus. \
+            new_method_call_message(
+                interface._remote_service_name,
+                interface._remote_object_path,
+                self.dbus_method.interface_name,
+                self.dbus_method.method_name,
+            )
 
         if args:
             new_call_message.append_data(
@@ -79,12 +87,15 @@ class DbusMethodAsyncBinded(DbusBindedAsync):
             new_call_message.send()
             return
 
-        reply_message = await self.interface.attached_bus.call_async(
+        reply_message = await interface._attached_bus.call_async(
             new_call_message)
         return reply_message.get_contents()
 
     def __call__(self, *args: Any, **kwargs: Any) -> Any:
-        if self.interface.is_binded:
+        interface = self.interface_ref()
+        assert interface is not None
+
+        if interface._is_binded:
 
             if len(args) == self.dbus_method.num_of_args:
                 assert not kwargs, (
@@ -100,15 +111,18 @@ class DbusMethodAsyncBinded(DbusBindedAsync):
             return self._call_dbus_async(*rebuilt_args)
         else:
             return self.dbus_method.original_method(
-                self.interface, *args, **kwargs)
+                interface, *args, **kwargs)
 
     async def _call_from_dbus(
             self,
             request_message: SdBusMessage) -> None:
+        interface = self.interface_ref()
+        assert interface is not None
+
         request_data = request_message.get_contents()
 
         local_method = self.dbus_method.original_method.__get__(
-            self.interface, None)
+            interface, None)
 
         try:
             if isinstance(request_data, tuple):
@@ -201,6 +215,8 @@ class DbusProperty(DbusSomethingAsync, Generic[T]):
         self.property_setter = property_setter
         self.flags = flags
 
+        self.__doc__ = property_getter.__doc__
+
 
 class DbusPropertyAsync(DbusProperty[T]):
     def __get__(self,
@@ -225,63 +241,77 @@ class DbusPropertyAsyncBinded(DbusBindedAsync):
                  dbus_property: DbusPropertyAsync[T],
                  interface: DbusInterfaceBaseAsync):
         self.dbus_property = dbus_property
-        self.interface = interface
+        self.interface_ref = weakref(interface)
+
+        self.__doc__ = dbus_property.__doc__
 
     def __await__(self) -> Generator[Any, None, T]:
         return self.get_async().__await__()
 
     async def get_async(self) -> T:
-        if not self.interface.is_binded:
-            return self.dbus_property.property_getter(
-                self.interface)
+        interface = self.interface_ref()
+        assert interface is not None
 
-        assert self.interface.attached_bus is not None
-        assert self.interface.remote_service_name is not None
-        assert self.interface.remote_object_path is not None
+        if not interface._is_binded:
+            return self.dbus_property.property_getter(
+                interface)
+
+        assert interface._attached_bus is not None
+        assert interface._remote_service_name is not None
+        assert interface._remote_object_path is not None
         assert self.dbus_property.property_name is not None
         assert self.dbus_property.interface_name is not None
-        new_call_message = self.interface.attached_bus. \
+        new_call_message = interface._attached_bus. \
             new_property_get_message(
-                self.interface.remote_service_name,
-                self.interface.remote_object_path,
+                interface._remote_service_name,
+                interface._remote_object_path,
                 self.dbus_property.interface_name,
                 self.dbus_property.property_name,
             )
 
-        reply_message = await self.interface.attached_bus. \
+        reply_message = await interface._attached_bus. \
             call_async(new_call_message)
         # Get method returns variant but we only need contents of variant
         return cast(T, reply_message.get_contents()[1])
 
     def _reply_get_sync(self, message: SdBusMessage) -> None:
-        reply_data: Any = self.dbus_property.property_getter(self.interface)
+        interface = self.interface_ref()
+        assert interface is not None
+
+        reply_data: Any = self.dbus_property.property_getter(interface)
         message.append_data(self.dbus_property.property_signature, reply_data)
 
     def _reply_set_sync(self, message: SdBusMessage) -> None:
+        interface = self.interface_ref()
+        assert interface is not None
+
         assert self.dbus_property.property_setter is not None
         data_to_set_to: Any = message.get_contents()
 
-        self.dbus_property.property_setter(self.interface, data_to_set_to)
+        self.dbus_property.property_setter(interface, data_to_set_to)
 
     async def set_async(self, complete_object: T) -> None:
-        if not self.interface.is_binded:
+        interface = self.interface_ref()
+        assert interface is not None
+
+        if not interface._is_binded:
             if self.dbus_property.property_setter is None:
                 raise ValueError('Property has no setter')
 
             self.dbus_property.property_setter(
-                self.interface, complete_object)
+                interface, complete_object)
 
             return
 
-        assert self.interface.attached_bus is not None
-        assert self.interface.remote_service_name is not None
-        assert self.interface.remote_object_path is not None
+        assert interface._attached_bus is not None
+        assert interface._remote_service_name is not None
+        assert interface._remote_object_path is not None
         assert self.dbus_property.property_name is not None
         assert self.dbus_property.interface_name is not None
-        new_call_message = self.interface.attached_bus. \
+        new_call_message = interface._attached_bus. \
             new_property_set_message(
-                self.interface.remote_service_name,
-                self.interface.remote_object_path,
+                interface._remote_service_name,
+                interface._remote_object_path,
                 self.dbus_property.interface_name,
                 self.dbus_property.property_name,
             )
@@ -289,7 +319,7 @@ class DbusPropertyAsyncBinded(DbusBindedAsync):
         new_call_message.append_data(
             'v', (self.dbus_property.property_signature, complete_object))
 
-        await self.interface.attached_bus.call_async(new_call_message)
+        await interface._attached_bus.call_async(new_call_message)
 
 
 def dbus_property_async(
@@ -338,17 +368,21 @@ def dbus_property_async(
 class DbusSignal(Generic[T], DbusSomethingAsync):
     def __init__(
         self,
+        original_function: Callable[[Any], T],
         signal_name: str,
         signature: str = "",
         args_names: Sequence[str] = (),
         flags: int = 0,
     ):
         super().__init__()
+        self.original_function = original_function
         self.signal_name = signal_name
         self.signature = signature
         self.args_names = args_names
         self.flags = flags
 
+        self.__doc__ = original_function.__doc__
+
     def __get__(self,
                 obj: DbusInterfaceBaseAsync,
                 obj_class: Optional[Type[DbusInterfaceBaseAsync]] = None,
@@ -361,18 +395,23 @@ class DbusSignalBinded(Generic[T], DbusBindedAsync):
                  dbus_signal: DbusSignal[T],
                  interface: DbusInterfaceBaseAsync):
         self.dbus_signal = dbus_signal
-        self.interface = interface
+        self.interface_ref = weakref(interface)
+
+        self.__doc__ = dbus_signal.__doc__
 
     async def _get_dbus_queue(self) -> Queue[SdBusMessage]:
-        assert self.interface.attached_bus is not None
-        assert self.interface.remote_service_name is not None
-        assert self.interface.remote_object_path is not None
+        interface = self.interface_ref()
+        assert interface is not None
+
+        assert interface._attached_bus is not None
+        assert interface._remote_service_name is not None
+        assert interface._remote_object_path is not None
         assert self.dbus_signal.interface_name is not None
         assert self.dbus_signal.signal_name is not None
 
-        return await self.interface.attached_bus.get_signal_queue_async(
-            self.interface.remote_service_name,
-            self.interface.remote_object_path,
+        return await interface._attached_bus.get_signal_queue_async(
+            interface._remote_service_name,
+            interface._remote_object_path,
             self.dbus_signal.interface_name,
             self.dbus_signal.signal_name,
         )
@@ -380,15 +419,21 @@ class DbusSignalBinded(Generic[T], DbusBindedAsync):
     def _cleanup_local_queue(
             self,
             queue_ref: weak_ref[Queue[T]]) -> None:
-        self.interface._local_signal_queues[self.dbus_signal].remove(queue_ref)
+        interface = self.interface_ref()
+        assert interface is not None
+
+        interface._local_signal_queues[self.dbus_signal].remove(queue_ref)
 
     def _get_local_queue(self) -> Queue[T]:
+        interface = self.interface_ref()
+        assert interface is not None
+
         try:
-            list_of_queues = self.interface._local_signal_queues[
+            list_of_queues = interface._local_signal_queues[
                 self.dbus_signal]
         except KeyError:
             list_of_queues = []
-            self.interface._local_signal_queues[
+            interface._local_signal_queues[
                 self.dbus_signal] = list_of_queues
 
         new_queue: Queue[T] = Queue()
@@ -399,7 +444,10 @@ class DbusSignalBinded(Generic[T], DbusBindedAsync):
 
     async def __aiter__(self) -> AsyncGenerator[T, None]:
 
-        if self.interface.is_binded:
+        interface = self.interface_ref()
+        assert interface is not None
+
+        if interface._is_binded:
             message_queue = await self._get_dbus_queue()
 
             while True:
@@ -413,13 +461,16 @@ class DbusSignalBinded(Generic[T], DbusBindedAsync):
                 yield next_data
 
     def _emit_message(self, args: T) -> None:
-        assert self.interface.attached_bus is not None
-        assert self.interface.serving_object_path is not None
+        interface = self.interface_ref()
+        assert interface is not None
+
+        assert interface._attached_bus is not None
+        assert interface._serving_object_path is not None
         assert self.dbus_signal.interface_name is not None
         assert self.dbus_signal.signal_name is not None
 
-        signal_message = self.interface.attached_bus.new_signal_message(
-            self.interface.serving_object_path,
+        signal_message = interface._attached_bus.new_signal_message(
+            interface._serving_object_path,
             self.dbus_signal.interface_name,
             self.dbus_signal.signal_name,
         )
@@ -434,12 +485,14 @@ class DbusSignalBinded(Generic[T], DbusBindedAsync):
         signal_message.send()
 
     def emit(self, args: T) -> None:
-        if self.interface.activated_interfaces:
+        interface = self.interface_ref()
+        assert interface is not None
+
+        if interface._activated_interfaces:
             self._emit_message(args)
 
         try:
-            list_of_queues = self.interface._local_signal_queues[
-                self.dbus_signal]
+            list_of_queues = interface._local_signal_queues[self.dbus_signal]
         except KeyError:
             return
 
@@ -474,6 +527,7 @@ def dbus_signal_async(
             )
 
         return DbusSignal(
+            pseudo_function,
             signal_name, signal_signature,
             signal_args_names, flags,
         )
@@ -582,12 +636,12 @@ class DbusInterfaceBaseAsync(metaclass=DbusInterfaceMetaAsync):
 
     # TODO: make all attributes start with _
     def __init__(self) -> None:
-        self.activated_interfaces: List[SdBusInterface] = []
-        self.is_binded: bool = False
-        self.remote_service_name: Optional[str] = None
-        self.remote_object_path: Optional[str] = None
-        self.attached_bus: Optional[SdBus] = None
-        self.serving_object_path: Optional[str] = None
+        self._activated_interfaces: List[SdBusInterface] = []
+        self._is_binded: bool = False
+        self._remote_service_name: Optional[str] = None
+        self._remote_object_path: Optional[str] = None
+        self._attached_bus: Optional[SdBus] = None
+        self._serving_object_path: Optional[str] = None
         self._local_signal_queues: \
             Dict[DbusSignal[Any], List[weak_ref[Queue[Any]]]] = {}
 
@@ -610,8 +664,8 @@ class DbusInterfaceBaseAsync(metaclass=DbusInterfaceMetaAsync):
         if bus is None:
             bus = get_default_bus()
         # TODO: Being able to serve multiple buses and object
-        self.attached_bus = bus
-        self.serving_object_path = object_path
+        self._attached_bus = bus
+        self._serving_object_path = object_path
         # TODO: can be optimized with a single loop
         interface_map: Dict[str, List[DbusBindedAsync]] = {}
 
@@ -683,7 +737,7 @@ class DbusInterfaceBaseAsync(metaclass=DbusInterfaceMetaAsync):
 
             bus.add_interface(new_interface, object_path,
                               interface_name)
-            self.activated_interfaces.append(new_interface)
+            self._activated_interfaces.append(new_interface)
 
     def _connect(
         self,
@@ -692,10 +746,10 @@ class DbusInterfaceBaseAsync(metaclass=DbusInterfaceMetaAsync):
         bus: Optional[SdBus] = None,
     ) -> None:
 
-        self.is_binded = True
-        self.attached_bus = bus if bus is not None else get_default_bus()
-        self.remote_service_name = service_name
-        self.remote_object_path = object_path
+        self._is_binded = True
+        self._attached_bus = bus if bus is not None else get_default_bus()
+        self._remote_service_name = service_name
+        self._remote_object_path = object_path
 
     @classmethod
     def new_connect(
@@ -738,6 +792,38 @@ class DbusIntrospectableAsync(
         raise NotImplementedError
 
 
+class DbusPropertiesInterfaceAsync(
+    DbusInterfaceBaseAsync,
+    interface_name='org.freedesktop.DBus.Properties',
+    serving_enabled=False,
+):
+    @dbus_signal_async('sa{sv}as')
+    def properties_changed(self) -> Tuple[str,
+                                          Dict[str, Tuple[str, Any]],
+                                          List[str]]:
+        ...
+
+
 class DbusInterfaceCommonAsync(
-        DbusPeerInterfaceAsync, DbusIntrospectableAsync):
+        DbusPeerInterfaceAsync, DbusPropertiesInterfaceAsync,
+        DbusIntrospectableAsync):
     ...
+
+
+class DbusObjectManagerInterfaceAsync(
+    DbusInterfaceCommonAsync,
+    interface_name='org.freedesktop.DBus.ObjectManager',
+    serving_enabled=False,
+):
+    @dbus_method_async(result_signature='a{oa{sa{sv}}}')
+    async def get_managed_objects(
+            self) -> Dict[str, Dict[str, Dict[str, Any]]]:
+        raise NotImplementedError
+
+    @dbus_signal_async('oa{sa{sv}}')
+    def interfaces_added(self) -> Tuple[str, Dict[str, Dict[str, Any]]]:
+        raise NotImplementedError
+
+    @dbus_signal_async('oao')
+    def interfaces_removed(self) -> Tuple[str, List[str]]:
+        raise NotImplementedError
