@@ -28,7 +28,7 @@ from sdbus import (DbusFailedError, DbusFileExistsError,
                    DbusInterfaceCommonAsync, DbusNoReplyFlag,
                    dbus_method_async, dbus_method_async_override,
                    dbus_property_async, dbus_property_async_override,
-                   dbus_signal_async)
+                   dbus_signal_async, DbusUnknownObjectError)
 
 from .common_test_util import TempDbusTest
 
@@ -72,6 +72,7 @@ class TestInterface(DbusInterfaceCommonAsync,
 
     @dbus_method_async("s", "s")
     async def upper(self, string: str) -> str:
+        """Uppercase the input"""
         return string.upper()
 
     @dbus_method_async(result_signature='x')
@@ -84,6 +85,7 @@ class TestInterface(DbusInterfaceCommonAsync,
 
     @dbus_property_async("s")
     def test_property(self) -> str:
+        """Test property"""
         return self.test_string
 
     @test_property.setter
@@ -116,6 +118,7 @@ class TestInterface(DbusInterfaceCommonAsync,
 
     @dbus_signal_async('ss')
     def test_signal(self) -> Tuple[str, str]:
+        """Test signal"""
         raise NotImplementedError
 
     @dbus_method_async()
@@ -365,3 +368,44 @@ class TestProxy(TempDbusTest):
     async def test_no_reply_method(self) -> None:
         await wait_for(self.test_object_connection.no_reply_method('yes'),
                        timeout=1)
+
+    async def test_interface_remove(self) -> None:
+        from gc import collect
+
+        del self.test_object
+
+        collect()
+
+        loop = get_running_loop()
+
+        t = loop.create_task(self.test_object_connection.dbus_introspect())
+
+        try:
+            await wait_for(t, timeout=0.2)
+        except DbusUnknownObjectError:
+            ...
+
+        self.assertRaises(DbusUnknownObjectError, t.result)
+
+    def test_docstring(self) -> None:
+        from pydoc import getdoc
+
+        with self.subTest('Method local doc'):
+            self.assertTrue(getdoc(self.test_object.upper))
+
+        with self.subTest('Method remote doc'):
+            self.assertTrue(getdoc(self.test_object_connection.upper))
+
+        with self.subTest('Property local doc'):
+            self.assertTrue(getdoc(self.test_object.test_property))
+
+        with self.subTest('Property remote doc'):
+            self.assertTrue(
+                getdoc(self.test_object_connection.test_property))
+
+        with self.subTest('Signal local doc'):
+            self.assertTrue(getdoc(self.test_object.test_signal))
+
+        with self.subTest('Signal remote doc'):
+            self.assertTrue(
+                getdoc(self.test_object_connection.test_signal))
